from datetime import datetime
from sqlalchemy import Column, DateTime, ForeignKey, Integer, String
from sqlalchemy.orm import relationship
from cs_ai_common.rds.entities.base import Base


class Location(Base):
    __tablename__ = "locations"

    id = Column(Integer(), primary_key=True)
    country = Column(String(50), nullable=False)
    city = Column(String(50), nullable=False)
    postal_code = Column(String(20), nullable=False)
    advertisements = relationship('Advertisement', back_populates='location')


from datetime import datetime, timezone
from sqlalchemy import Boolean, Column, DateTime, ForeignKey, Integer, String
from sqlalchemy.orm import relationship
from cs_ai_common.rds.entities.base import Base


class OfferInstance(Base):
    __tablename__ = "offer_instances"

    id = Column(Integer(), primary_key=True)
    status = Column(String(50), nullable=False)
    created_at = Column(DateTime(), default=datetime.now(timezone.utc))
    external_url = Column(String(50), nullable=False)
    views = Column(Integer(), default=0)
    offer_id = Column(Integer(), ForeignKey('offers.id'))

    offer = relationship('Offer', back_populates='offer_instances')


from .advertisement import Advertisement
from .location import Location
from .offer import Offer
from .offer_instances import OfferInstance
from .user import User
from .external_site import ExternalSite
from .external_site_instance import ExternalSiteInstance




from datetime import datetime, timezone
from sqlalchemy import Boolean, Column, DateTime, ForeignKey, Integer, String
from sqlalchemy.orm import relationship
from cs_ai_common.rds.entities.base import Base


class User(Base):
    __tablename__ = "users"

    id = Column(Integer(), primary_key=True)
    email = Column(String(50), nullable=False)
    password = Column(String(50), nullable=False)
    confirmed_password = Column(Boolean(), nullable=False)
    created_at = Column(DateTime(), default=datetime.now(timezone.utc))
    number_of_ads = Column(Integer(), default=0)
    first_name = Column(String(50), nullable=False)
    last_name = Column(String(50), nullable=False)
    refresh_token = Column(String(), nullable=True)

    offers = relationship('Offer', back_populates='user')
    external_site_instances = relationship('ExternalSiteInstance', back_populates='user')



from datetime import datetime, timezone
from sqlalchemy import Boolean, Column, DateTime, ForeignKey, Integer, String
from sqlalchemy.orm import relationship
from cs_ai_common.rds.entities.base import Base


class ExternalSiteInstance(Base):
    __tablename__ = "external_site_instances"

    id = Column(Integer(), primary_key=True)
    auth_type = Column(String(50), nullable=False)
    auth_creds = Column(String(200), nullable=False)

    user_id = Column(Integer(), ForeignKey('users.id'))
    user = relationship('User', back_populates='external_site_instances')
    
    external_site_id = Column(Integer(), ForeignKey('external_site.id'))
    external_site = relationship('ExternalSite', back_populates='external_site_instances')



from datetime import datetime
from sqlalchemy import Column, DateTime, ForeignKey, Integer, String
from sqlalchemy.orm import relationship
from cs_ai_common.rds.entities.base import Base

class Offer(Base):
    __tablename__ = "offers"
    
    id = Column(Integer(), primary_key=True)
    make = Column(String(50), nullable=False)
    model = Column(String(50), nullable=False)
    generation = Column(String(50), nullable=False)
    production_year = Column(Integer(), nullable=False)
    engine_capacity = Column(Integer(), nullable=False)
    engine_power = Column(Integer(), nullable=False)
    mileage = Column(Integer(), nullable=False)
    transmission = Column(String(30), nullable=False)
    fuel_type = Column(String(30), nullable=False)
    thumbnails_folder_url = Column(String(200), nullable=False)
    price = Column(Integer(), nullable=False)
    price_currency = Column(String(20), nullable=False)
    created_at = Column(DateTime(), default=datetime.now)
    location = Column(String(50), nullable=False)
    equipment = Column(String(), nullable=False)
    description = Column(String(), nullable=False)
    title = Column(String(), nullable=False)

    user = relationship('User', back_populates='offers')
    user_id = Column(Integer(), ForeignKey('users.id'))

    offer_instances = relationship('OfferInstance', back_populates='offer')


from datetime import datetime
from sqlalchemy import Column, DateTime, ForeignKey, Integer, String
from sqlalchemy.orm import relationship
from cs_ai_common.rds.entities.base import Base

class Advertisement(Base):
    __tablename__ = "advertisements"
    
    id = Column(Integer(), primary_key=True)
    make = Column(String(50), nullable=False)
    model = Column(String(50), nullable=False)
    production_year = Column(Integer(), nullable=False)
    engine_capacity = Column(Integer(), nullable=False)
    engine_power = Column(Integer(), nullable=False)
    mileage = Column(Integer(), nullable=False)
    transmission = Column(String(30), nullable=False)
    fuel_type = Column(String(30), nullable=False)
    cache_hash = Column(String(100), nullable=False)
    source = Column(String(30), nullable=False)
    link = Column(String(200), nullable=False)
    thumbnail = Column(String(200), nullable=False)
    price = Column(Integer(), nullable=False)
    price_currency = Column(String(20), nullable=False)
    location_code = Column(String(20), nullable=False)
    created_at = Column(DateTime(), default=datetime.now)
    location_id = Column(Integer(), ForeignKey('locations.id'))
    location = relationship('Location', back_populates='advertisements')



from datetime import datetime, timezone
from sqlalchemy import Boolean, Column, DateTime, ForeignKey, Integer, String
from sqlalchemy.orm import relationship
from cs_ai_common.rds.entities.base import Base


class ExternalSite(Base):
    __tablename__ = "external_site"

    id = Column(Integer(), primary_key=True)
    name = Column(String(50), nullable=False)
    logo_url = Column(String(200), nullable=False)
    base_url = Column(String(200), nullable=False)

    external_site_instances = relationship('ExternalSiteInstance', back_populates='external_site')


from cs_ai_common.dynamodb.task_table import update_task_status_failed
from cs_ai_common.logging.internal_logger import InternalLogger


def startup_app(callback, retry_on, retries: int, raw_event: dict = None):
    try:
        return callback()
    except retry_on as e:
        InternalLogger.LogDebug(f"Attenpting to retry {callback.__name__} due to {e}")
        if retries > 0:
            InternalLogger.LogDebug(f"Retrying {callback.__name__} due to {e}")
            return startup_app(callback, retry_on=retry_on, retries=retries - 1, raw_event=raw_event)
        else:
            InternalLogger.LogError(f"Failed to run {callback.__name__} after retries. Last error: {e}")
            _update_task_status_failed(raw_event)
            raise                                               
    except Exception as e:
        InternalLogger.LogError(f"Failed to run {callback.__name__}. Error: {e}")
        _update_task_status_failed(raw_event)
        raise e
        
def _update_task_status_failed(raw_event: dict):
    task_id = raw_event["task_id"]
    update_task_status_failed(task_id)


import os
from typing import Tuple

from cs_ai_common.typings.proxy import ProxyProviders


class ProxyApiClient:
    _username: str
    _password: str
    _base_proxy_url: str

    def __init__(self, proxy_provider: ProxyProviders):
        self._username, self._password, self._base_proxy_url = self.get_proxy_auth(proxy_provider)

    def get_proxy_ip(self) -> str:
        return self._base_proxy_url.format(self._username, self._password)
    
    def get_proxy_auth(self, provider: str) -> Tuple[str, str, str]:
        """
        Retrieves proxy authentication credentials from environment variables.
        Args:
            prefix (str): The prefix used to identify the environment variables.
        Returns:
            Tuple[str, str, str]: A tuple containing the username, password, and base proxy URL.
        Raises:
            ValueError: If any of the required environment variables are missing.
        """

        _username = os.getenv(f'{provider}_USERNAME', None)
        _password = os.getenv(f'{provider}_PASSWORD', None)
        _base_proxy_url = os.getenv(f'{provider}_BASE_URL', None)

        if any([_username is None, _password is None, _base_proxy_url is None]):
            raise ValueError('Missing proxy credentials')
        
        return _username, _password, _base_proxy_url